@@ -44,10 +44,10 @@ extern crate nalgebra as na;
 extern crate num_traits;
 
 use std::cmp::Ordering;
-use std::fmt::{Debug, Display, Formatter, Result as FmtResult};
+use std::fmt::{Debug, Display, Formatter, LowerExp, Result as FmtResult};
 use std::iter::{Product, Sum};
 use std::num::FpCategory;
-use std::ops::{Add, AddAssign, Deref, DerefMut, Div, DivAssign, Mul, MulAssign, Neg, Rem, Sub, SubAssign};
+use std::ops::{Add, AddAssign, Deref, DerefMut, Div, DivAssign, Mul, MulAssign, Neg, Rem, RemAssign, Sub, SubAssign};
 
 pub use num_traits::{Float, FloatConst, Num, One, Zero};
 
@@ -60,7 +60,7 @@ pub mod linalg;
 
 use num_traits::{FromPrimitive, Inv, MulAdd, MulAddAssign, NumCast, Pow, Signed, ToPrimitive, Unsigned};
 
-use na::{SVector, Scalar};
+use na::{Const, OVector, SVector, Scalar};
 
 // Re-export traits useful for construction and extension of duals
 pub use na::allocator::Allocator;
@@ -77,26 +77,35 @@ pub use na::{DefaultAllocator, Dim, DimName};
 ///
 /// Lastly, the `Rem` remainder operator is not correctly or fully defined for `Dual`, and will panic.
 #[derive(Clone, Copy)]
-pub struct Hyperdual<T: Copy + Scalar, const N: usize>(SVector<T, N>);
+pub struct OHyperdual<T: Copy + Scalar, N: Dim + DimName>(OVector<T, N>)
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy;
 
-impl<T: Copy + Scalar, const N: usize> Hyperdual<T, N> {
+pub type Hyperdual<T, const N: usize> = OHyperdual<T, Const<N>>;
+
+impl<T: Copy + Scalar, N: Dim + DimName> OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     /// Create a new dual number from its real and dual parts.
     #[inline]
-    pub fn from_slice(v: &[T]) -> Hyperdual<T, N> {
-        Hyperdual(SVector::<T, N>::from_row_slice(v))
+    pub fn from_slice(v: &[T]) -> Self {
+        Self(OVector::<T, N>::from_row_slice(v))
     }
 
     /// Create a new dual number from a real number.
     ///
     /// The dual part is set to zero.
     #[inline]
-    pub fn from_real(real: T) -> Hyperdual<T, N>
+    pub fn from_real(real: T) -> Self
     where
         T: Zero,
     {
-        let mut dual = SVector::<T, N>::zeros();
+        let mut dual = OVector::<T, N>::zeros();
         dual[0] = real;
-        Hyperdual(dual)
+        Self(dual)
     }
 
     /// Returns the real part
@@ -117,27 +126,31 @@ impl<T: Copy + Scalar, const N: usize> Hyperdual<T, N> {
     }
 
     #[inline]
-    pub fn map_dual<F>(&self, r: T, f: F) -> Hyperdual<T, N>
+    pub fn map_dual<F>(&self, r: T, f: F) -> Self
     where
         F: Fn(&T) -> T,
     {
         // TODO: improve, so the real does not get mapped
         let mut v = self.map(|x| f(&x));
         v[0] = r;
-        Hyperdual(v)
+        Self(v)
     }
 
     /// Create a new dual number from a function
     #[inline]
-    pub fn from_fn<F>(mut f: F) -> Hyperdual<T, N>
+    pub fn from_fn<F>(mut f: F) -> Self
     where
         F: FnMut(usize) -> T,
     {
-        Hyperdual(SVector::<T, N>::from_fn(|i, _| f(i)))
+        Self(OVector::<T, N>::from_fn(|i, _| f(i)))
     }
 }
 
-impl<T: Copy + Scalar, const N: usize> Debug for Hyperdual<T, N> {
+impl<T: Copy + Scalar, N: Dim + DimName> Debug for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     fn fmt(&self, f: &mut Formatter) -> FmtResult {
         let mut a = f.debug_tuple("Dual");
         for x in self.iter() {
@@ -147,51 +160,79 @@ impl<T: Copy + Scalar, const N: usize> Debug for Hyperdual<T, N> {
     }
 }
 
-impl<T: Copy + Scalar + Num + Zero, const N: usize> Default for Hyperdual<T, N> {
+impl<T: Copy + Scalar + Num + Zero, N: Dim + DimName> Default for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     #[inline]
-    fn default() -> Hyperdual<T, N> {
-        Hyperdual::zero()
+    fn default() -> Self {
+        Self::zero()
     }
 }
 
-impl<T: Copy + Scalar + Zero, const N: usize> From<T> for Hyperdual<T, N> {
+impl<T: Copy + Scalar + Zero, N: Dim + DimName> From<T> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     #[inline]
-    fn from(real: T) -> Hyperdual<T, N> {
-        Hyperdual::from_real(real)
+    fn from(real: T) -> Self {
+        Self::from_real(real)
     }
 }
 
-impl<T: Copy + Scalar, const N: usize> Deref for Hyperdual<T, N> {
-    type Target = SVector<T, N>;
+impl<T: Copy + Scalar, N: Dim + DimName> Deref for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
+    type Target = OVector<T, N>;
 
     #[inline]
-    fn deref(&self) -> &SVector<T, N> {
+    fn deref(&self) -> &OVector<T, N> {
         &self.0
     }
 }
 
-impl<T: Copy + Scalar, const N: usize> DerefMut for Hyperdual<T, N> {
+impl<T: Copy + Scalar, N: Dim + DimName> DerefMut for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     #[inline]
-    fn deref_mut(&mut self) -> &mut SVector<T, N> {
+    fn deref_mut(&mut self) -> &mut OVector<T, N> {
         &mut self.0
     }
 }
 
-impl<T: Copy + Scalar, const N: usize> AsRef<SVector<T, N>> for Hyperdual<T, N> {
+impl<T: Copy + Scalar, N: Dim + DimName> AsRef<OVector<T, N>> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     #[inline]
-    fn as_ref(&self) -> &SVector<T, N> {
+    fn as_ref(&self) -> &OVector<T, N> {
         &self.0
     }
 }
 
-impl<T: Copy + Scalar, const N: usize> AsMut<SVector<T, N>> for Hyperdual<T, N> {
+impl<T: Copy + Scalar, N: Dim + DimName> AsMut<OVector<T, N>> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     #[inline]
-    fn as_mut(&mut self) -> &mut SVector<T, N> {
+    fn as_mut(&mut self) -> &mut OVector<T, N> {
         &mut self.0
     }
 }
 
-impl<T: Copy + Scalar + Neg<Output = T>, const N: usize> Hyperdual<T, N> {
+impl<T: Copy + Scalar + Neg<Output = T>, N: Dim + DimName> OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     /// Returns the conjugate of the dual number.
     #[inline]
     pub fn conjugate(self) -> Self {
@@ -199,7 +240,11 @@ impl<T: Copy + Scalar + Neg<Output = T>, const N: usize> Hyperdual<T, N> {
     }
 }
 
-impl<T: Copy + Scalar + Display, const N: usize> Display for Hyperdual<T, N> {
+impl<T: Copy + Scalar + Display, N: Dim + DimName> Display for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     fn fmt(&self, f: &mut Formatter) -> FmtResult {
         let precision = f.precision().unwrap_or(4);
 
@@ -212,14 +257,39 @@ impl<T: Copy + Scalar + Display, const N: usize> Display for Hyperdual<T, N> {
     }
 }
 
-impl<T: Copy + Scalar + PartialEq, const N: usize> PartialEq<Self> for Hyperdual<T, N> {
+impl<T: Copy + Scalar + LowerExp, N: Dim + DimName> LowerExp for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
+    fn fmt(&self, f: &mut Formatter) -> FmtResult {
+        let precision = f.precision().unwrap_or(4);
+
+        write!(f, "{:.p$e}", self.real(), p = precision)?;
+        for (i, x) in self.iter().skip(1).enumerate() {
+            write!(f, " + {:.p$e}\u{03B5}{}", x, i + 1, p = precision)?;
+        }
+
+        Ok(())
+    }
+}
+
+impl<T: Copy + Scalar + PartialEq, N: Dim + DimName> PartialEq<Self> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     #[inline]
     fn eq(&self, rhs: &Self) -> bool {
         self.0 == rhs.0
     }
 }
 
-impl<T: Copy + Scalar + PartialOrd, const N: usize> PartialOrd<Self> for Hyperdual<T, N> {
+impl<T: Copy + Scalar + PartialOrd, N: Dim + DimName> PartialOrd<Self> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     #[inline]
     fn partial_cmp(&self, rhs: &Self) -> Option<Ordering> {
         PartialOrd::partial_cmp(self.real_ref(), rhs.real_ref())
@@ -236,14 +306,22 @@ impl<T: Copy + Scalar + PartialOrd, const N: usize> PartialOrd<Self> for Hyperdu
     }
 }
 
-impl<T: Copy + Scalar + PartialEq, const N: usize> PartialEq<T> for Hyperdual<T, N> {
+impl<T: Copy + Scalar + PartialEq, N: Dim + DimName> PartialEq<T> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     #[inline]
     fn eq(&self, rhs: &T) -> bool {
         *self.real_ref() == *rhs
     }
 }
 
-impl<T: Copy + Scalar + PartialOrd, const N: usize> PartialOrd<T> for Hyperdual<T, N> {
+impl<T: Copy + Scalar + PartialOrd, N: Dim + DimName> PartialOrd<T> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     #[inline]
     fn partial_cmp(&self, rhs: &T) -> Option<Ordering> {
         PartialOrd::partial_cmp(self.real_ref(), rhs)
@@ -262,7 +340,11 @@ impl<T: Copy + Scalar + PartialOrd, const N: usize> PartialOrd<T> for Hyperdual<
 
 macro_rules! impl_to_primitive {
     ($($name:ident, $ty:ty),*) => {
-        impl<T: Copy + Scalar + ToPrimitive, const N: usize> ToPrimitive for Hyperdual<T, N>
+        impl<T: Copy + Scalar + ToPrimitive, N: Dim + DimName> ToPrimitive for OHyperdual<T, N>
+        where
+            DefaultAllocator: Allocator<T, N>,
+            Owned<T, N>: Copy,
+
 {
             $(
                 #[inline]
@@ -276,14 +358,16 @@ macro_rules! impl_to_primitive {
 
 macro_rules! impl_from_primitive {
     ($($name:ident, $ty:ty),*) => {
-        impl<T: Copy + Scalar + FromPrimitive, const N: usize> FromPrimitive for Hyperdual<T, N>
+        impl<T: Copy + Scalar + FromPrimitive, N: Dim + DimName> FromPrimitive for OHyperdual<T, N>
             where
-                T: Zero
+                T: Zero,
+                DefaultAllocator: Allocator<T, N>,
+                Owned<T, N>: Copy,
         {
             $(
                 #[inline]
-                fn $name(n: $ty) -> Option<Hyperdual<T,N>> {
-                    T::$name(n).map(Hyperdual::from_real)
+                fn $name(n: $ty) -> Option<OHyperdual<T,N>> {
+                    T::$name(n).map(Self::from_real)
                 }
             )*
         }
@@ -312,116 +396,198 @@ impl_primitive_cast! {
     to_f64,     from_f64    - f64
 }
 
-impl<T: Copy + Scalar + Num, const N: usize> Add<T> for Hyperdual<T, N> {
-    type Output = Hyperdual<T, N>;
+impl<T: Copy + Scalar + Num, N: Dim + DimName> Add<T> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
+    type Output = OHyperdual<T, N>;
 
     #[inline]
-    fn add(self, rhs: T) -> Hyperdual<T, N> {
+    fn add(self, rhs: T) -> Self {
         let mut d = self;
         d[0] = d[0] + rhs;
         d
     }
 }
 
-impl<T: Copy + Scalar + Num, const N: usize> AddAssign<T> for Hyperdual<T, N> {
+impl<T: Copy + Scalar + Num, N: Dim + DimName> AddAssign<T> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     #[inline]
     fn add_assign(&mut self, rhs: T) {
-        *self = (*self) + Hyperdual::from_real(rhs)
+        *self = (*self) + Self::from_real(rhs)
     }
 }
 
-impl<T: Copy + Scalar + Num, const N: usize> Sub<T> for Hyperdual<T, N> {
-    type Output = Hyperdual<T, N>;
+impl<T: Copy + Scalar + Num, N: Dim + DimName> Sub<T> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
+    type Output = OHyperdual<T, N>;
 
     #[inline]
-    fn sub(self, rhs: T) -> Hyperdual<T, N> {
+    fn sub(self, rhs: T) -> Self {
         let mut d = self;
         d[0] = d[0] - rhs;
         d
     }
 }
 
-impl<T: Copy + Scalar + Num, const N: usize> SubAssign<T> for Hyperdual<T, N> {
+impl<T: Copy + Scalar + Num, N: Dim + DimName> SubAssign<T> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     #[inline]
     fn sub_assign(&mut self, rhs: T) {
-        *self = (*self) - Hyperdual::from_real(rhs)
+        *self = (*self) - Self::from_real(rhs)
     }
 }
 
-impl<T: Copy + Scalar + Num, const N: usize> Mul<T> for Hyperdual<T, N> {
-    type Output = Hyperdual<T, N>;
+impl<T: Copy + Scalar + Num, N: Dim + DimName> Mul<T> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
+    type Output = OHyperdual<T, N>;
 
     #[inline]
-    fn mul(self, rhs: T) -> Hyperdual<T, N> {
-        self * Hyperdual::from_real(rhs)
+    fn mul(self, rhs: T) -> Self {
+        self * Self::from_real(rhs)
     }
 }
 
-impl<T: Copy + Scalar + Num, const N: usize> MulAssign<T> for Hyperdual<T, N> {
+impl<T: Copy + Scalar + Num, N: Dim + DimName> MulAssign<T> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     #[inline]
     fn mul_assign(&mut self, rhs: T) {
-        *self = (*self) * Hyperdual::from_real(rhs)
+        *self = (*self) * Self::from_real(rhs)
     }
 }
 
-impl<T: Copy + Scalar + Num, const N: usize> Div<T> for Hyperdual<T, N> {
-    type Output = Hyperdual<T, N>;
+impl<T: Copy + Scalar + Num, N: Dim + DimName> Div<T> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
+    type Output = OHyperdual<T, N>;
 
     #[inline]
-    fn div(self, rhs: T) -> Hyperdual<T, N> {
-        self / Hyperdual::from_real(rhs)
+    fn div(self, rhs: T) -> Self {
+        self / Self::from_real(rhs)
     }
 }
 
-impl<T: Copy + Scalar + Num, const N: usize> DivAssign<T> for Hyperdual<T, N> {
+impl<T: Copy + Scalar + Num, N: Dim + DimName> DivAssign<T> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     #[inline]
     fn div_assign(&mut self, rhs: T) {
         *self = (*self) / Self::from_real(rhs)
     }
 }
 
-impl<T: Copy + Scalar + Signed, const N: usize> Neg for Hyperdual<T, N> {
+impl<T: Copy + Scalar + Signed, N: Dim + DimName> Neg for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     type Output = Self;
 
     #[inline]
     fn neg(self) -> Self {
-        Hyperdual(self.map(|x| x.neg()))
+        Self(self.map(|x| x.neg()))
     }
 }
 
-impl<T: Copy + Scalar + Num, const N: usize> Add<Self> for Hyperdual<T, N> {
+impl<T: Copy + Scalar + Num, N: Dim + DimName> Add<Self> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     type Output = Self;
 
     #[inline]
     fn add(self, rhs: Self) -> Self {
-        Hyperdual(self.zip_map(&rhs, |x, y| x + y))
+        Self(self.zip_map(&rhs, |x, y| x + y))
     }
 }
 
-impl<T: Copy + Scalar + Num, const N: usize> AddAssign<Self> for Hyperdual<T, N> {
+impl<T: Copy + Scalar + Num, N: Dim + DimName> Add<&Self> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
+    type Output = Self;
+
+    #[inline]
+    fn add(self, rhs: &Self) -> Self {
+        Self(self.zip_map(rhs, |x, y| x + y))
+    }
+}
+
+impl<T: Copy + Scalar + Num, N: Dim + DimName> AddAssign<Self> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     #[inline]
     fn add_assign(&mut self, rhs: Self) {
         *self = (*self) + rhs
     }
 }
 
-impl<T: Copy + Scalar + Num, const N: usize> Sub<Self> for Hyperdual<T, N> {
+impl<T: Copy + Scalar + Num, N: Dim + DimName> Sub<Self> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     type Output = Self;
 
     #[inline]
     fn sub(self, rhs: Self) -> Self {
-        Hyperdual(self.zip_map(&rhs, |x, y| x - y))
+        Self(self.zip_map(&rhs, |x, y| x - y))
     }
 }
 
-impl<T: Copy + Scalar + Num, const N: usize> SubAssign<Self> for Hyperdual<T, N> {
+impl<T: Copy + Scalar + Num, N: Dim + DimName> Sub<&Self> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
+    type Output = Self;
+
+    #[inline]
+    fn sub(self, rhs: &Self) -> Self {
+        Self(self.zip_map(rhs, |x, y| x - y))
+    }
+}
+
+impl<T: Copy + Scalar + Num, N: Dim + DimName> SubAssign<Self> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     #[inline]
     fn sub_assign(&mut self, rhs: Self) {
         *self = (*self) - rhs
     }
 }
 
-impl<T: Copy + Scalar + Num, const N: usize> Mul<Self> for Hyperdual<T, N> {
+impl<T: Copy + Scalar + Num, N: Dim + DimName> Mul<Self> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     type Output = Self;
 
     #[allow(clippy::suspicious_arithmetic_impl)]
@@ -429,11 +595,31 @@ impl<T: Copy + Scalar + Num, const N: usize> Mul<Self> for Hyperdual<T, N> {
     fn mul(self, rhs: Self) -> Self {
         let mut v = self.zip_map(&rhs, |x, y| rhs.real() * x + self.real() * y);
         v[0] = self.real() * rhs.real();
-        Hyperdual(v)
+        Self(v)
     }
 }
 
-impl<T: Copy + Scalar + Num, const N: usize> MulAssign<Self> for Hyperdual<T, N> {
+impl<T: Copy + Scalar + Num, N: Dim + DimName> Mul<&Self> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
+    type Output = Self;
+
+    #[allow(clippy::suspicious_arithmetic_impl)]
+    #[inline]
+    fn mul(self, rhs: &Self) -> Self {
+        let mut v = self.zip_map(rhs, |x, y| rhs.real() * x + self.real() * y);
+        v[0] = self.real() * rhs.real();
+        Self(v)
+    }
+}
+
+impl<T: Copy + Scalar + Num, N: Dim + DimName> MulAssign<Self> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     #[inline]
     fn mul_assign(&mut self, rhs: Self) {
         *self = (*self) * rhs
@@ -443,8 +629,12 @@ impl<T: Copy + Scalar + Num, const N: usize> MulAssign<Self> for Hyperdual<T, N>
 macro_rules! impl_mul_add {
     ($(<$a:ident, $b:ident>),*) => {
         $(
-            impl<T: Copy + Scalar + Num + Mul + Add, const N: usize> MulAdd<$a, $b> for Hyperdual<T,N> {
-                type Output = Hyperdual<T,N>;
+            impl<T: Copy + Scalar + Num + Mul + Add, N: Dim + DimName> MulAdd<$a, $b> for OHyperdual<T,N>
+            where
+                DefaultAllocator: Allocator<T, N>,
+                Owned<T, N>: Copy,
+            {
+                type Output = OHyperdual<T,N>;
 
                 #[inline]
                 fn mul_add(self, a: $a, b: $b) -> Self {
@@ -452,7 +642,11 @@ macro_rules! impl_mul_add {
                 }
             }
 
-            impl<T: Copy + Scalar + Num + Mul + Add, const N: usize> MulAddAssign<$a, $b> for Hyperdual<T,N> {
+            impl<T: Copy + Scalar + Num + Mul + Add, N: Dim + DimName> MulAddAssign<$a, $b> for OHyperdual<T,N>
+            where
+                DefaultAllocator: Allocator<T, N>,
+                Owned<T, N>: Copy,
+            {
                 #[inline]
                 fn mul_add_assign(&mut self, a: $a, b: $b) {
                     *self = (*self * a) + b;
@@ -469,7 +663,11 @@ impl_mul_add! {
     <T, T>
 }
 
-impl<T: Copy + Scalar + Num, const N: usize> Div<Self> for Hyperdual<T, N> {
+impl<T: Copy + Scalar + Num, N: Dim + DimName> Div<Self> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     type Output = Self;
 
     #[inline]
@@ -479,18 +677,44 @@ impl<T: Copy + Scalar + Num, const N: usize> Div<Self> for Hyperdual<T, N> {
 
         let mut v = self.zip_map(&rhs, |x, y| (rhs.real() * x - self.real() * y) / d);
         v[0] = self.real() / rhs.real();
-        Hyperdual(v)
+        Self(v)
     }
 }
 
-impl<T: Copy + Scalar + Num, const N: usize> DivAssign<Self> for Hyperdual<T, N> {
+impl<T: Copy + Scalar + Num, N: Dim + DimName> Div<&Self> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
+    type Output = Self;
+
+    #[inline]
+    #[allow(clippy::suspicious_arithmetic_impl)]
+    fn div(self, rhs: &Self) -> Self {
+        let d = rhs.real() * rhs.real();
+
+        let mut v = self.zip_map(rhs, |x, y| (rhs.real() * x - self.real() * y) / d);
+        v[0] = self.real() / rhs.real();
+        Self(v)
+    }
+}
+
+impl<T: Copy + Scalar + Num, N: Dim + DimName> DivAssign<Self> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     #[inline]
     fn div_assign(&mut self, rhs: Self) {
         *self = (*self) / rhs
     }
 }
 
-impl<T: Copy + Scalar + Num, const N: usize> Rem<Self> for Hyperdual<T, N> {
+impl<T: Copy + Scalar + Num, N: Dim + DimName> Rem<Self> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     type Output = Self;
 
     /// **UNIMPLEMENTED!!!**
@@ -502,9 +726,33 @@ impl<T: Copy + Scalar + Num, const N: usize> Rem<Self> for Hyperdual<T, N> {
     }
 }
 
-impl<T: Copy + Scalar, P: Into<Hyperdual<T, N>>, const N: usize> Pow<P> for Hyperdual<T, N>
+impl<T: Copy + Scalar + Num, N: Dim + DimName> Rem<&Self> for OHyperdual<T, N>
 where
-    Hyperdual<T, N>: Float,
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
+    type Output = Self;
+
+    fn rem(self, _: &Self) -> Self {
+        unimplemented!()
+    }
+}
+
+impl<T: Copy + Scalar + Num, N: Dim + DimName> RemAssign<Self> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
+    fn rem_assign(&mut self, _: Self) {
+        unimplemented!()
+    }
+}
+
+impl<T: Copy + Scalar, P: Into<OHyperdual<T, N>>, N: Dim + DimName> Pow<P> for OHyperdual<T, N>
+where
+    OHyperdual<T, N>: Float,
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
 {
     type Output = Self;
 
@@ -514,9 +762,11 @@ where
     }
 }
 
-impl<T: Copy + Scalar, const N: usize> Inv for Hyperdual<T, N>
+impl<T: Copy + Scalar, N: Dim + DimName> Inv for OHyperdual<T, N>
 where
     Self: One + Div<Output = Self>,
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
 {
     type Output = Self;
 
@@ -526,14 +776,16 @@ where
     }
 }
 
-impl<T: Copy + Scalar, const N: usize> Signed for Hyperdual<T, N>
+impl<T: Copy + Scalar, N: Dim + DimName> Signed for OHyperdual<T, N>
 where
     T: Signed + PartialOrd,
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
 {
     #[inline]
     fn abs(&self) -> Self {
         let s = self.real().signum();
-        Hyperdual(self.map(|x| x * s))
+        Self(self.map(|x| x * s))
     }
 
     #[inline]
@@ -561,12 +813,22 @@ where
     }
 }
 
-impl<T: Copy + Scalar + Unsigned, const N: usize> Unsigned for Hyperdual<T, N> where Self: Num {}
+impl<T: Copy + Scalar + Unsigned, N: Dim + DimName> Unsigned for OHyperdual<T, N>
+where
+    Self: Num,
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
+}
 
-impl<T: Copy + Scalar + Num + Zero, const N: usize> Zero for Hyperdual<T, N> {
+impl<T: Copy + Scalar + Num + Zero, N: Dim + DimName> Zero for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     #[inline]
-    fn zero() -> Hyperdual<T, N> {
-        Hyperdual::from_real(T::zero())
+    fn zero() -> Self {
+        Self::from_real(T::zero())
     }
 
     #[inline]
@@ -575,10 +837,14 @@ impl<T: Copy + Scalar + Num + Zero, const N: usize> Zero for Hyperdual<T, N> {
     }
 }
 
-impl<T: Copy + Scalar + Num + One, const N: usize> One for Hyperdual<T, N> {
+impl<T: Copy + Scalar + Num + One, N: Dim + DimName> One for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     #[inline]
-    fn one() -> Hyperdual<T, N> {
-        Hyperdual::from_real(T::one())
+    fn one() -> Self {
+        Self::from_real(T::one())
     }
 
     #[inline]
@@ -590,18 +856,26 @@ impl<T: Copy + Scalar + Num + One, const N: usize> One for Hyperdual<T, N> {
     }
 }
 
-impl<T: Copy + Scalar + Num, const N: usize> Num for Hyperdual<T, N> {
+impl<T: Copy + Scalar + Num, N: Dim + DimName> Num for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     type FromStrRadixErr = <T as Num>::FromStrRadixErr;
 
     #[inline]
-    fn from_str_radix(str: &str, radix: u32) -> Result<Hyperdual<T, N>, Self::FromStrRadixErr> {
+    fn from_str_radix(str: &str, radix: u32) -> Result<OHyperdual<T, N>, Self::FromStrRadixErr> {
         <T as Num>::from_str_radix(str, radix).map(Self::from_real)
     }
 }
 
-impl<T: Copy + Scalar + Float, const N: usize> NumCast for Hyperdual<T, N> {
+impl<T: Copy + Scalar + Float, N: Dim + DimName> NumCast for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     #[inline]
-    fn from<P: ToPrimitive>(n: P) -> Option<Hyperdual<T, N>> {
+    fn from<P: ToPrimitive>(n: P) -> Option<OHyperdual<T, N>> {
         <T as NumCast>::from(n).map(Self::from_real)
     }
 }
@@ -609,12 +883,16 @@ impl<T: Copy + Scalar + Float, const N: usize> NumCast for Hyperdual<T, N> {
 macro_rules! impl_float_const {
     ($($c:ident),*) => {
         $(
-            fn $c() -> Hyperdual<T, N> { Self::from_real(T::$c()) }
+            fn $c()-> Self { Self::from_real(T::$c()) }
         )*
     }
 }
 
-impl<T: Copy + Scalar + FloatConst + Zero, const N: usize> FloatConst for Hyperdual<T, N> {
+impl<T: Copy + Scalar + FloatConst + Zero, N: Dim + DimName> FloatConst for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
     impl_float_const!(
         E,
         FRAC_1_PI,
@@ -678,38 +956,56 @@ macro_rules! impl_boolean_op {
 macro_rules! impl_real_op {
     ($($op:ident),*) => {
         $(
-            fn $op(self) -> Self { Hyperdual::from_real(self.real().$op()) }
+            fn $op(self) -> Self { Self::from_real(self.real().$op()) }
         )*
     }
 }
 
-impl<T: Copy + Scalar + Num + Zero, const N: usize> Sum for Hyperdual<T, N> {
-    fn sum<I: Iterator<Item = Hyperdual<T, N>>>(iter: I) -> Hyperdual<T, N> {
-        iter.fold(Hyperdual::zero(), |a, b| a + b)
+impl<T: Copy + Scalar + Num + Zero, N: Dim + DimName> Sum for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
+    fn sum<I: Iterator<Item = OHyperdual<T, N>>>(iter: I) -> Self {
+        iter.fold(Self::zero(), |a, b| a + b)
     }
 }
 
-impl<'a, T: Copy + Scalar + Num + Zero, const N: usize> Sum<&'a Hyperdual<T, N>> for Hyperdual<T, N> {
-    fn sum<I: Iterator<Item = &'a Hyperdual<T, N>>>(iter: I) -> Hyperdual<T, N> {
-        iter.fold(Hyperdual::zero(), |a, b| a + *b)
+impl<'a, T: Copy + Scalar + Num + Zero, N: Dim + DimName> Sum<&'a OHyperdual<T, N>> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
+    fn sum<I: Iterator<Item = &'a OHyperdual<T, N>>>(iter: I) -> Self {
+        iter.fold(Self::zero(), |a, b| a + *b)
     }
 }
 
-impl<T: Copy + Scalar + Num + One, const N: usize> Product for Hyperdual<T, N> {
-    fn product<I: Iterator<Item = Hyperdual<T, N>>>(iter: I) -> Hyperdual<T, N> {
-        iter.fold(Hyperdual::one(), |a, b| a * b)
+impl<T: Copy + Scalar + Num + One, N: Dim + DimName> Product for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
+    fn product<I: Iterator<Item = OHyperdual<T, N>>>(iter: I) -> Self {
+        iter.fold(Self::one(), |a, b| a * b)
     }
 }
 
-impl<'a, T: Copy + Scalar + Num + One, const N: usize> Product<&'a Hyperdual<T, N>> for Hyperdual<T, N> {
-    fn product<I: Iterator<Item = &'a Hyperdual<T, N>>>(iter: I) -> Hyperdual<T, N> {
-        iter.fold(Hyperdual::one(), |a, b| a * *b)
+impl<'a, T: Copy + Scalar + Num + One, N: Dim + DimName> Product<&'a OHyperdual<T, N>> for OHyperdual<T, N>
+where
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
+{
+    fn product<I: Iterator<Item = &'a OHyperdual<T, N>>>(iter: I) -> Self {
+        iter.fold(Self::one(), |a, b| a * *b)
     }
 }
 
-impl<T: Copy + Scalar, const N: usize> Float for Hyperdual<T, N>
+impl<T: Copy + Scalar, N: Dim + DimName> Float for OHyperdual<T, N>
 where
     T: Float + Signed + FloatConst,
+    DefaultAllocator: Allocator<T, N>,
+    Owned<T, N>: Copy,
 {
     impl_real_constant!(nan, infinity, neg_infinity, neg_zero, min_positive_value, epsilon, min_value, max_value);
 
@@ -744,7 +1040,7 @@ where
     #[inline]
     fn abs(self) -> Self {
         let s = self.real().signum();
-        Hyperdual(self.map(|x| x * s))
+        Self(self.map(|x| x * s))
     }
 
     #[inline]
@@ -776,7 +1072,7 @@ where
 
     #[inline]
     fn mul_add(self, a: Self, b: Self) -> Self {
-        let mut dual = Hyperdual::from_real(self.real().mul_add(a.real(), b.real()));
+        let mut dual = Self::from_real(self.real().mul_add(a.real(), b.real()));
 
         for x in 1..self.len() {
             dual[x] = self[x] * a.real() + self.real() * a[x] + b[x];
@@ -806,7 +1102,7 @@ where
 
         let mut v = self.zip_map(&n, |x, y| a * x + b * y);
         v[0] = real_part;
-        Hyperdual(v)
+        Self(v)
     }
 
     #[inline]
@@ -859,7 +1155,7 @@ where
         let c = self.real().hypot(other.real());
         let mut v = self.zip_map(&other, |x, y| (self.real() * x + other.real() * y) / c);
         v[0] = c;
-        Hyperdual(v)
+        Self(v)
     }
 
     #[inline]
@@ -907,7 +1203,7 @@ where
         let c = self.real().powi(2) + other.real().powi(2);
         let mut v = self.zip_map(&other, |x, y| (other.real() * x - self.real() * y) / c);
         v[0] = self.real().atan2(other.real());
-        Hyperdual(v)
+        Self(v)
     }
 
     #[inline]
@@ -1032,3 +1328,33 @@ pub fn vector_from_hyperspace<T: Scalar + Zero + Float, const DIM_VECTOR: usize,
 ) -> SVector<T, DIM_VECTOR> {
     x_dual.map(|x| x.real())
 }
+
+pub fn hyperspace_from_ovector<T: Copy + Scalar + Num + Zero, D: Dim + DimName, N: Dim + DimName>(v: &OVector<T, N>) -> OVector<OHyperdual<T, D>, N>
+where
+    DefaultAllocator: Allocator<T, D> + Allocator<T, N> + Allocator<OHyperdual<T, D>, N>,
+    Owned<T, D>: Copy,
+{
+    let mut space_slice = vec![OHyperdual::<T, D>::zero(); N::dim()];
+    for i in 0..N::dim() {
+        space_slice[i] = OHyperdual::<T, D>::from_fn(|j| {
+            if j == 0 {
+                v[i]
+            } else if i + 1 == j {
+                T::one()
+            } else {
+                T::zero()
+            }
+        });
+    }
+    OVector::<OHyperdual<T, D>, N>::from_row_slice(&space_slice)
+}
+
+pub fn ovector_from_hyperspace<T: Scalar + Zero + Float, DimVector: Dim + DimName, DimHyper: Dim + DimName>(
+    x_dual: &OVector<OHyperdual<T, DimHyper>, DimVector>,
+) -> OVector<T, DimVector>
+where
+    DefaultAllocator: Allocator<T, DimVector> + Allocator<OHyperdual<T, DimHyper>, DimVector> + Allocator<T, DimHyper>,
+    <DefaultAllocator as Allocator<T, DimHyper>>::Buffer: Copy,
+{
+    x_dual.map(|x| x.real())
+}
